# -*- coding: utf-8 -*-
'''
    :codeauthor: Lukas Raska <lukas@raska.me>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import elasticsearch
from salt.exceptions import CommandExecutionError

# Import elasticsearch exceptions
NO_ELASTIC = False
try:
    from elasticsearch import TransportError, NotFoundError
except Exception:
    NO_ELASTIC = True


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(NO_ELASTIC, 'Install elasticsearch-py before running Elasticsearch unit tests.')
class ElasticsearchTestCase(TestCase):
    '''
    Test cases for salt.modules.elasticsearch
    '''
    @staticmethod
    def es_return_true(hosts=None, profile=None):
        return True

    @staticmethod
    def es_raise_command_execution_error(hosts=None, profile=None):
        raise CommandExecutionError("custom message")

    # 'ping' function tests: 2

    def test_ping(self):
        '''
        Test if ping succeeds
        '''
        with patch.object(elasticsearch, '_get_instance', self.es_return_true):
            self.assertTrue(elasticsearch.ping())

    def test_ping_failure(self):
        '''
        Test if ping fails
        '''
        with patch.object(elasticsearch, '_get_instance', self.es_raise_command_execution_error):
            self.assertFalse(elasticsearch.ping())

    # 'info' function tests: 2

    def test_info(self):
        '''
        Test if status fetch succeeds
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def info(self):
                '''
                Mock of info method
                '''
                return [{"test": "key"}]

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertListEqual(elasticsearch.info(), [{"test": "key"}])

    def test_info_failure(self):
        '''
        Test if status fetch fails with CommandExecutionError
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def info(self):
                '''
                Mock of info method
                '''
                raise TransportError("custom error", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.info)

    # 'node_info' function tests: 2

    def test_node_info(self):
        '''
        Test if node status fetch succeeds
        '''
        class MockElasticNodes(object):
            '''
            Mock of Elasticsearch NodesClient
            '''
            def info(self, node_id=None, flat_settings=None):
                '''
                Mock of info method
                '''
                return [{"test": "key"}]

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            nodes = MockElasticNodes()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertListEqual(elasticsearch.node_info(), [{"test": "key"}])

    def test_node_info_failure(self):
        '''
        Test if node status fetch fails with CommandExecutionError
        '''
        class MockElasticNodes(object):
            '''
            Mock of Elasticsearch NodesClient
            '''
            def info(self, node_id=None, flat_settings=None):
                '''
                Mock of info method
                '''
                raise TransportError("custom error", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            nodes = MockElasticNodes()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.node_info)

    # 'cluster_health' function tests: 2

    def test_cluster_health(self):
        '''
        Test if cluster status health fetch succeeds
        '''
        class MockElasticCluster(object):
            '''
            Mock of Elasticsearch ClusterClient
            '''
            def health(self, index=None, level=None, local=None):
                '''
                Mock of health method
                '''
                return [{"test": "key"}]

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            cluster = MockElasticCluster()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertListEqual(elasticsearch.cluster_health(), [{"test": "key"}])

    def test_cluster_health_failure(self):
        '''
        Test if cluster status health fetch fails with CommandExecutionError
        '''
        class MockElasticCluster(object):
            '''
            Mock of Elasticsearch ClusterClient
            '''
            def health(self, index=None, level=None, local=None):
                '''
                Mock of health method
                '''
                raise TransportError("custom error", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            cluster = MockElasticCluster()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.cluster_health)

    # 'cluster_stats' function tests: 2

    def test_cluster_stats(self):
        '''
        Test if cluster stats fetch succeeds
        '''
        class MockElasticCluster(object):
            '''
            Mock of Elasticsearch ClusterClient
            '''
            def stats(self, node_id=None):
                '''
                Mock of health method
                '''
                return [{"test": "key"}]

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            cluster = MockElasticCluster()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertListEqual(elasticsearch.cluster_stats(), [{"test": "key"}])

    def test_cluster_stats_failure(self):
        '''
        Test if cluster stats fetch fails with CommandExecutionError
        '''
        class MockElasticCluster(object):
            '''
            Mock of Elasticsearch ClusterClient
            '''
            def stats(self, node_id=None):
                '''
                Mock of health method
                '''
                raise TransportError("custom error", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            cluster = MockElasticCluster()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.cluster_stats)

    # 'alias_create' function tests: 3

    def test_alias_create(self):
        '''
        Test if alias is created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_alias(self, index=None, name=None, body=None):
                '''
                Mock of put_alias method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.alias_create("foo", "bar", body="baz"))

    def test_alias_create_unack(self):
        '''
        Test if alias creation is not acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_alias(self, index=None, name=None, body=None):
                '''
                Mock of put_alias method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.alias_create("foo", "bar", body="baz"))

    def test_alias_create_failure(self):
        '''
        Test if alias creation fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_alias(self, index=None, name=None, body=None):
                '''
                Mock of put_alias method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.alias_create, "foo", "bar", body="baz")

    # 'alias_delete' function tests: 3

    def test_alias_delete(self):
        '''
        Test if alias is deleted
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_alias(self, index=None, name=None):
                '''
                Mock of delete_alias method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.alias_delete("foo", "bar"))

    def test_alias_delete_unack(self):
        '''
        Test if alias deletion is not acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_alias(self, index=None, name=None):
                '''
                Mock of delete_alias method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.alias_delete("foo", "bar"))

    def test_alias_delete_failure(self):
        '''
        Test if alias deletion fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_alias(self, index=None, name=None):
                '''
                Mock of delete_alias method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.alias_delete, "foo", "bar")

    # 'alias_exists' function tests: 3

    def test_alias_exists(self):
        '''
        Test if alias exists
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_alias(self, index=None, name=None):
                '''
                Mock of exists_alias method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.alias_exists("foo", "bar"))

    def test_alias_exists_not(self):
        '''
        Test if alias doesn't exist
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_alias(self, index=None, name=None):
                '''
                Mock of exists_alias method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.alias_exists("foo", "bar"))

    def test_alias_exists_failure(self):
        '''
        Test if alias status obtain fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_alias(self, index=None, name=None):
                '''
                Mock of exists_alias method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.alias_exists, "foo", "bar")

    # 'alias_get' function tests: 3

    def test_alias_get(self):
        '''
        Test if alias can be obtained
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_alias(self, index=None, name=None):
                '''
                Mock of get_alias method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.alias_get("foo", "bar"), {"test": "key"})

    def test_alias_get_not(self):
        '''
        Test if alias doesn't exist
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_alias(self, index=None, name=None):
                '''
                Mock of get_alias method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.alias_get("foo", "bar"), None)

    def test_alias_get_failure(self):
        '''
        Test if alias obtain fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_alias(self, index=None, name=None):
                '''
                Mock of get_alias method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.alias_get, "foo", "bar")

    # 'document_create' function tests: 2

    def test_document_create(self):
        '''
        Test if document can be created
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def index(self, index=None, doc_type=None, body=None, id=None):
                '''
                Mock of index method
                '''
                return {"test": "key"}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.document_create("foo", "bar"), {"test": "key"})

    def test_document_create_failure(self):
        '''
        Test if document creation fails with CommandExecutionError
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def index(self, index=None, doc_type=None, body=None, id=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.document_create, "foo", "bar")

    # 'document_delete' function tests: 2

    def test_document_delete(self):
        '''
        Test if document can be deleted
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                return {"test": "key"}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.document_delete("foo", "bar", "baz"), {"test": "key"})

    def test_document_delete_failure(self):
        '''
        Test if document deletion fails with CommandExecutionError
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.document_delete, "foo", "bar", "baz")

    # 'document_exists' function tests: 3

    def test_document_exists(self):
        '''
        Test if document status can be obtained
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def exists(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                return True

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.document_exists("foo", "bar"))

    def test_document_exists_not(self):
        '''
        Test if document doesn't exist
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def exists(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                return False

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.document_exists("foo", "bar"))

    def test_document_exists_failure(self):
        '''
        Test if document exist state fails with CommandExecutionError
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def exists(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.document_exists, "foo", "bar")

    # 'document_get' function tests: 3

    def test_document_get(self):
        '''
        Test if document exists
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                return {"test": "key"}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.document_get("foo", "bar"), {"test": "key"})

    def test_document_get_not(self):
        '''
        Test if document doesn't exit
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                raise NotFoundError

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.document_get("foo", "bar"), None)

    def test_document_get_failure(self):
        '''
        Test if document obtain fails with CommandExecutionError
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get(self, index=None, doc_type=None, id=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.document_get, "foo", "bar")

    # 'index_create' function tests: 5

    def test_index_create(self):
        '''
        Test if index can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def create(self, index=None, body=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True, "shards_acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_create("foo", "bar"))

    def test_index_create_no_shards(self):
        '''
        Test if index is created and no shards info is returned
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def create(self, index=None, body=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_create("foo", "bar"))

    def test_index_create_not_shards(self):
        '''
        Test if index is created and shards didn't acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def create(self, index=None, body=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True, "shards_acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_create("foo", "bar"))

    def test_index_create_not(self):
        '''
        Test if index is created and shards didn't acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def create(self, index=None, body=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": False, "shards_acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_create("foo", "bar"))

    def test_index_create_failure(self):
        '''
        Test if index creation fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def create(self, index=None, body=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_create, "foo", "bar")

    # 'index_delete' function tests: 3

    def test_index_delete(self):
        '''
        Test if index can be deleted
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete(self, index=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_delete("foo", "bar"))

    def test_index_delete_not(self):
        '''
        Test if index is deleted and shards didn't acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete(self, index=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_delete("foo", "bar"))

    def test_index_delete_failure(self):
        '''
        Test if index deletion fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete(self, index=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_delete, "foo", "bar")

    # 'index_exists' function tests: 3

    def test_index_exists(self):
        '''
        Test if index exists
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists(self, index=None):
                '''
                Mock of index method
                '''
                return True

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_exists("foo", "bar"))

    def test_index_exists_not(self):
        '''
        Test if index doesn't exist
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists(self, index=None):
                '''
                Mock of index method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_exists("foo", "bar"))

    def test_index_exists_failure(self):
        '''
        Test if alias exist state fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists(self, index=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_exists, "foo", "bar")

    # 'index_get' function tests: 3

    def test_index_get(self):
        '''
        Test if index can be obtained
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get(self, index=None):
                '''
                Mock of index method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.index_get("foo", "bar"), {"test": "key"})

    def test_index_get_not(self):
        '''
        Test if index doesn't exist
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get(self, index=None):
                '''
                Mock of index method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.index_get("foo", "bar"), None)

    def test_index_get_failure(self):
        '''
        Test if index obtain fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get(self, index=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_get, "foo", "bar")

    # 'index_open' function tests: 3

    def test_index_open(self):
        '''
        Test if index can be opened
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def open(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_open("foo", "bar"))

    def test_index_open_not(self):
        '''
        Test if index open isn't acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def open(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_open("foo", "bar"))

    def test_index_open_failure(self):
        '''
        Test if alias opening fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def open(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_open, "foo", "bar")

    # 'index_close' function tests: 3

    def test_index_close(self):
        '''
        Test if index can be closed
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def close(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_close("foo", "bar"))

    def test_index_close_not(self):
        '''
        Test if index close isn't acked
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def close(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_close("foo", "bar"))

    def test_index_close_failure(self):
        '''
        Test if index closing fails with CommandExecutionError
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def close(self, index=None, allow_no_indices=None, expand_wildcards=None, ignore_unavailable=None):
                '''
                Mock of index method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_close, "foo", "bar")

    # 'mapping_create' function tests: 3

    def test_mapping_create(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_mapping(self, index=None, doc_type=None, body=None):
                '''
                Mock of put_mapping method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.mapping_create("foo", "bar", "baz"))

    def test_mapping_create_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_mapping(self, index=None, doc_type=None, body=None):
                '''
                Mock of put_mapping method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.mapping_create("foo", "bar", "baz"))

    def test_mapping_create_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_mapping(self, index=None, doc_type=None, body=None):
                '''
                Mock of put_mapping method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.mapping_create, "foo", "bar", "baz")

    # 'mapping_delete' function tests: 3

    def test_mapping_delete(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_mapping(self, index=None, doc_type=None):
                '''
                Mock of put_mapping method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.mapping_delete("foo", "bar", "baz"))

    def test_mapping_delete_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_mapping(self, index=None, doc_type=None):
                '''
                Mock of put_mapping method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.mapping_delete("foo", "bar", "baz"))

    def test_mapping_delete_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_mapping(self, index=None, doc_type=None):
                '''
                Mock of put_mapping method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.mapping_delete, "foo", "bar", "baz")

    # 'mapping_get' function tests: 3

    def test_mapping_get(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_mapping(self, index=None, doc_type=None):
                '''
                Mock of get_mapping method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.mapping_get("foo", "bar", "baz"), {"test": "key"})

    def test_mapping_get_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_mapping(self, index=None, doc_type=None):
                '''
                Mock of get_mapping method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.mapping_get("foo", "bar", "baz"), None)

    def test_mapping_get_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_mapping(self, index=None, doc_type=None):
                '''
                Mock of get_mapping method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.mapping_get, "foo", "bar", "baz")

    # 'index_template_create' function tests: 3

    def test_index_template_create(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_template(self, name=None, body=None):
                '''
                Mock of put_template method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_template_create("foo", "bar"))

    def test_index_template_create_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_template(self, name=None, body=None):
                '''
                Mock of put_template method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_template_create("foo", "bar"))

    def test_index_template_create_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_template(self, name=None, body=None):
                '''
                Mock of put_template method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_template_create, "foo", "bar")

    # 'index_template_delete' function tests: 3

    def test_index_template_delete(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_template(self, name=None):
                '''
                Mock of delete_template method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_template_delete("foo"))

    def test_index_template_delete_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_template(self, name=None):
                '''
                Mock of delete_template method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_template_delete("foo"))

    def test_index_template_delete_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_template(self, name=None):
                '''
                Mock of delete_template method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_template_delete, "foo")

    # 'index_template_exists' function tests: 3

    def test_index_template_exists(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_template(self, name=None):
                '''
                Mock of exists_template method
                '''
                return True

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.index_template_exists("foo"))

    def test_index_template_exists_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_template(self, name=None):
                '''
                Mock of exists_template method
                '''
                return False

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.index_template_exists("foo"))

    def test_index_template_exists_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def exists_template(self, name=None):
                '''
                Mock of exists_template method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_template_exists, "foo")

    # 'index_template_get' function tests: 3

    def test_index_template_get(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_template(self, name=None):
                '''
                Mock of get_template method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.index_template_get("foo"), {"test": "key"})

    def test_index_template_get_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_template(self, name=None):
                '''
                Mock of get_template method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.index_template_get("foo"), None)

    def test_index_template_get_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIndices(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_template(self, name=None):
                '''
                Mock of get_template method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            indices = MockElasticIndices()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.index_template_get, "foo")

    # 'pipeline_get' function tests: 4

    def test_pipeline_get(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_pipeline(self, id=None):
                '''
                Mock of get_pipeline method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.pipeline_get("foo"), {"test": "key"})

    def test_pipeline_get_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_pipeline(self, id=None):
                '''
                Mock of get_pipeline method
                '''
                raise NotFoundError

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.pipeline_get("foo"), None)

    def test_pipeline_get_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def get_pipeline(self, id=None):
                '''
                Mock of get_pipeline method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_get, "foo")

    def test_pipeline_get_wrong_version(self):
        '''
        Test if mapping creation fails with CEE on invalid elasticsearch-py version
        '''
        class MockElasticIngest(object):
            pass

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_get, "foo")

    # 'pipeline_delete' function tests: 4

    def test_pipeline_delete(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_pipeline(self, id=None):
                '''
                Mock of delete_pipeline method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.pipeline_delete("foo"))

    def test_pipeline_delete_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_pipeline(self, id=None):
                '''
                Mock of delete_pipeline method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.pipeline_delete("foo"))

    def test_pipeline_delete_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def delete_pipeline(self, id=None):
                '''
                Mock of delete_pipeline method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_delete, "foo")

    def test_pipeline_delete_wrong_version(self):
        '''
        Test if mapping creation fails with CEE on invalid elasticsearch-py version
        '''
        class MockElasticIngest(object):
            pass

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_delete, "foo")

    # 'pipeline_create' function tests: 4

    def test_pipeline_create(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_pipeline(self, id=None, body=None):
                '''
                Mock of put_pipeline method
                '''
                return {"acknowledged": True}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.pipeline_create("foo", "bar"))

    def test_pipeline_create_not(self):
        '''
        Test if mapping creation didn't ack
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_pipeline(self, id=None, body=None):
                '''
                Mock of put_pipeline method
                '''
                return {"acknowledged": False}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.pipeline_create("foo", "bar"))

    def test_pipeline_create_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def put_pipeline(self, id=None, body=None):
                '''
                Mock of put_pipeline method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_create, "foo", "bar")

    def test_pipeline_create_wrong_version(self):
        '''
        Test if mapping creation fails with CEE on invalid elasticsearch-py version
        '''
        class MockElasticIngest(object):
            pass

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_create, "foo", "bar")

    # 'pipeline_simulate' function tests: 3

    def test_pipeline_simulate(self):
        '''
        Test if mapping can be created
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def simulate(self, id=None, body=None, verbose=None):
                '''
                Mock of simulate method
                '''
                return {"test": "key"}

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.pipeline_simulate("foo", "bar"), {"test": "key"})

    def test_pipeline_simulate_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElasticIngest(object):
            '''
            Mock of Elasticsearch IndicesClient
            '''
            def simulate(self, id=None, body=None, verbose=None):
                '''
                Mock of simulate method
                '''
                raise TransportError("custom message", 123)

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_simulate, "foo", "bar")

    def test_pipeline_simulate_wrong_version(self):
        '''
        Test if mapping creation fails with CEE on invalid elasticsearch-py version
        '''
        class MockElasticIngest(object):
            pass

        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            ingest = MockElasticIngest()

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.pipeline_simulate, "foo", "bar")

    # 'search_template_get' function tests: 3

    def test_search_template_get(self):
        '''
        Test if mapping can be created
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get_template(self, id=None):
                '''
                Mock of get_template method
                '''
                return {"test": "key"}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertDictEqual(elasticsearch.search_template_get("foo"), {"test": "key"})

    def test_search_template_get_not(self):
        '''
        Test if mapping can be created
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get_template(self, id=None):
                '''
                Mock of get_template method
                '''
                raise NotFoundError

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertIs(elasticsearch.search_template_get("foo"), None)

    def test_search_template_get_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def get_template(self, id=None):
                '''
                Mock of get_template method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.search_template_get, "foo")

    # 'search_template_create' function tests: 3

    def test_search_template_create(self):
        '''
        Test if mapping can be created
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def put_template(self, id=None, body=None):
                '''
                Mock of put_template method
                '''
                return {"acknowledged": True}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.search_template_create("foo", "bar"))

    def test_search_template_create_not(self):
        '''
        Test if mapping can be created
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def put_template(self, id=None, body=None):
                '''
                Mock of put_template method
                '''
                return {"acknowledged": False}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.search_template_create("foo", "bar"))

    def test_search_template_create_failure(self):
        '''
        Test if mapping creation fails
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def put_template(self, id=None, body=None):
                '''
                Mock of put_template method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.search_template_create, "foo", "bar")

    # 'search_template_delete' function tests: 4

    def test_search_template_delete(self):
        '''
        Test if mapping can be deleted
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete_template(self, id=None):
                '''
                Mock of delete_template method
                '''
                return {"acknowledged": True}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.search_template_delete("foo"))

    def test_search_template_delete_not(self):
        '''
        Test if mapping can be deleted but not acked
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete_template(self, id=None):
                '''
                Mock of delete_template method
                '''
                return {"acknowledged": False}

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertFalse(elasticsearch.search_template_delete("foo"))

    def test_search_template_delete_not_exists(self):
        '''
        Test if deleting mapping doesn't exist
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete_template(self, id=None):
                '''
                Mock of delete_template method
                '''
                raise NotFoundError

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertTrue(elasticsearch.search_template_delete("foo"))

    def test_search_template_delete_failure(self):
        '''
        Test if mapping deletion fails
        '''
        class MockElastic(object):
            '''
            Mock of Elasticsearch client
            '''
            def delete_template(self, id=None):
                '''
                Mock of delete_template method
                '''
                raise TransportError("custom message", 123)

        with patch.object(elasticsearch, '_get_instance',
                          MagicMock(return_value=MockElastic())):
            self.assertRaises(CommandExecutionError, elasticsearch.search_template_delete, "foo")



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.guestfs as guestfs


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GuestfsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.guestfs
    '''
    def setup_loader_modules(self):
        return {guestfs: {}}

    # 'mount' function tests: 1
    def test_mount(self):
        '''
        Test if it mount an image
        '''
        with patch('os.path.join', MagicMock(return_value=True)), \
                patch('os.path.isdir', MagicMock(return_value=True)), \
                patch('os.listdir', MagicMock(return_value=False)), \
                patch.dict(guestfs.__salt__, {'cmd.run': MagicMock(return_value='')}):
            self.assertTrue(guestfs.mount('/srv/images/fedora.qcow'))



# -*- coding: utf-8 -*-
'''
Tests for the librato returner
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt Testing libs
from tests.support.case import ShellCase

# Import salt libs
from salt.returners import librato_return

log = logging.getLogger(__name__)

# JOBS DIR and FILES
MOCK_RET_OBJ = {
    "fun_args": [],
    "return": {
        "test-return-state": {
            "comment": "insertcommenthere",
            "name": "test-state-1",
            "start_time": "01: 19: 51.105566",
            "result": True,
            "duration": 3.645,
            "__run_num__": 193,
            "changes": {},
            "__id__": "test-return-state"
        },
        "test-return-state2": {
            "comment": "insertcommenthere",
            "name": "test-state-2",
            "start_time": "01: 19: 51.105566",
            "result": False,
            "duration": 3.645,
            "__run_num__": 194,
            "changes": {},
            "__id__": "test-return-state"
        }
    },
    "retcode": 2,
    "success": True,
    "fun": "state.highstate",
    "id": "Librato-Test",
    "out": "highstate"
}


class libratoTest(ShellCase):
    '''
    Test the librato returner
    '''

    def test_count_runtimes(self):
        '''
        Test the calculations
        '''
        results = librato_return._calculate_runtimes(MOCK_RET_OBJ['return'])
        self.assertEqual(results['num_failed_states'], 1)
        self.assertEqual(results['num_passed_states'], 1)
        self.assertEqual(results['runtime'], 7.29)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.sdb as sdb


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SdbTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.sdb
    '''
    def setup_loader_modules(self):
        return {sdb: {}}

    # 'get' function tests: 1

    def test_get(self):
        '''
        Test if it gets a value from a db, using a uri in the form of
        sdb://<profile>/<key>
        '''
        self.assertEqual(sdb.get('sdb://salt/foo'), 'sdb://salt/foo')

    # 'set_' function tests: 1

    def test_set(self):
        '''
        Test if it sets a value from a db, using a uri in the form of
        sdb://<profile>/<key>
        '''
        self.assertFalse(sdb.set_('sdb://mymemcached/foo', 'bar'))



# -*- coding: utf-8 -*-
'''
Icinga2 Common Utils
=================

This module provides common functionality for icinga2 module and state.

.. versionadded:: 2018.8.3
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging
import re

# Import Salt libs
import salt.utils.path

log = logging.getLogger(__name__)


def get_certs_path():
    icinga2_output = __salt__['cmd.run_all']([salt.utils.path.which('icinga2'), "--version"], python_shell=False)
    version = re.search(r'r\d+\.\d+', icinga2_output['stdout']).group(0)
    # Return new certs path for icinga2 >= 2.8
    if int(version.split('.')[1]) >= 8:
        return '/var/lib/icinga2/certs/'
    # Keep backwords compatibility with older icinga2
    return '/etc/icinga2/pki/'



# -*- coding: utf-8 -*-
'''
Unit tests for the docker module
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    Mock,
    NO_MOCK,
    NO_MOCK_REASON,
    patch
)
import logging
log = logging.getLogger(__name__)

# Import Salt Libs
import salt.config
import salt.loader
from salt.ext.six.moves import range
from salt.exceptions import CommandExecutionError
import salt.modules.dockermod as docker_mod


def _docker_py_version():
    try:
        if docker_mod.HAS_DOCKER_PY:
            return docker_mod.docker.version_info
    except AttributeError:
        pass
    return (0,)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(docker_mod.HAS_DOCKER_PY is False, 'docker-py must be installed to run these tests. Skipping.')
class DockerTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Validate docker module
    '''
    def setup_loader_modules(self):
        utils = salt.loader.utils(
            salt.config.DEFAULT_MINION_OPTS,
            whitelist=['args', 'docker', 'json', 'state', 'thin',
                       'systemd', 'path', 'platform']
        )
        return {docker_mod: {'__context__': {'docker.docker_version': ''},
                             '__utils__': utils}}

    try:
        docker_version = docker_mod.docker.version_info
    except AttributeError:
        docker_version = 0,

    def setUp(self):
        '''
        Ensure we aren't persisting context dunders between tests
        '''
        docker_mod.__context__.pop('docker.client', None)

    def test_failed_login(self):
        '''
        Check that when docker.login failed a retcode other then 0
        is part of the return.
        '''
        client = Mock()
        get_client_mock = MagicMock(return_value=client)
        ref_out = {
            'stdout': '',
            'stderr': 'login failed',
            'retcode': 1
        }
        with patch.dict(docker_mod.__pillar__, {'docker-registries': {'portus.example.com:5000':
                {'username': 'admin', 'password': 'linux12345', 'email': 'tux@example.com'}}}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                with patch.dict(docker_mod.__salt__, {'cmd.run_all': MagicMock(return_value=ref_out)}):
                    ret = docker_mod.login('portus.example.com:5000')
                    self.assertIn('retcode', ret)
                    self.assertNotEqual(ret['retcode'], 0)

    def test_ps_with_host_true(self):
        '''
        Check that docker.ps called with host is ``True``,
        include resutlt of ``network.interfaces`` command in returned result.
        '''
        client = Mock()
        client.containers = MagicMock(return_value=[])
        get_client_mock = MagicMock(return_value=client)
        network_interfaces = Mock(return_value={'mocked': None})

        with patch.dict(docker_mod.__salt__,
                        {'network.interfaces': network_interfaces}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                ret = docker_mod.ps_(host=True)
                self.assertEqual(ret,
                                 {'host': {'interfaces': {'mocked': None}}})

    def test_ps_with_filters(self):
        '''
        Check that docker.ps accept filters parameter.
        '''
        client = Mock()
        client.containers = MagicMock(return_value=[])
        get_client_mock = MagicMock(return_value=client)

        with patch.object(docker_mod, '_get_client', get_client_mock):
            docker_mod.ps_(filters={'label': 'KEY'})
            client.containers.assert_called_once_with(
                all=True,
                filters={'label': 'KEY'})

    def test_check_mine_cache_is_refreshed_on_container_change_event(self):
        '''
        Every command that might modify docker containers state.
        Should trig an update on ``mine.send``
        '''
        with patch.object(docker_mod, '_get_exec_driver'):
            client_args_mock = MagicMock(return_value={
                'create_container': [
                    'image', 'command', 'hostname', 'user', 'detach', 'stdin_open',
                    'tty', 'ports', 'environment', 'volumes', 'network_disabled',
                    'name', 'entrypoint', 'working_dir', 'domainname', 'cpuset',
                    'host_config', 'mac_address', 'labels', 'volume_driver',
                    'stop_signal', 'networking_config', 'healthcheck',
                    'stop_timeout'],
               'host_config': [
                   'binds', 'port_bindings', 'lxc_conf', 'publish_all_ports',
                   'links', 'privileged', 'dns', 'dns_search', 'volumes_from',
                   'network_mode', 'restart_policy', 'cap_add', 'cap_drop',
                   'devices', 'extra_hosts', 'read_only', 'pid_mode', 'ipc_mode',
                   'security_opt', 'ulimits', 'log_config', 'mem_limit',
                   'memswap_limit', 'mem_reservation', 'kernel_memory',
                   'mem_swappiness', 'cgroup_parent', 'group_add', 'cpu_quota',
                   'cpu_period', 'blkio_weight', 'blkio_weight_device',
                   'device_read_bps', 'device_write_bps', 'device_read_iops',
                   'device_write_iops', 'oom_kill_disable', 'shm_size', 'sysctls',
                   'tmpfs', 'oom_score_adj', 'dns_opt', 'cpu_shares',
                   'cpuset_cpus', 'userns_mode', 'pids_limit', 'isolation',
                   'auto_remove', 'storage_opt'],
               'networking_config': [
                   'aliases', 'links', 'ipv4_address', 'ipv6_address',
                   'link_local_ips'],
               }

            )

            for command_name, args in (('create', ()),
                                       ('rm_', ()),
                                       ('kill', ()),
                                       ('pause', ()),
                                       ('signal_', ('KILL',)),
                                       ('start_', ()),
                                       ('stop', ()),
                                       ('unpause', ()),
                                       ('_run', ('command',)),
                                       ('_script', ('command',)),
                                       ):
                mine_send = Mock()
                command = getattr(docker_mod, command_name)
                client = MagicMock()
                client.api_version = '1.12'
                with patch.dict(docker_mod.__salt__,
                                {'mine.send': mine_send,
                                 'container_resource.run': MagicMock(),
                                 'config.get': MagicMock(return_value=True),
                                 'cp.cache_file': MagicMock(return_value=False)}):
                    with patch.dict(docker_mod.__utils__,
                                    {'docker.get_client_args': client_args_mock}):
                        with patch.object(docker_mod, '_get_client', client):
                            command('container', *args)
                mine_send.assert_called_with('docker.ps', verbose=True, all=True,
                                             host=True)

    def test_update_mine(self):
        '''
        Test the docker.update_mine config option
        '''
        def config_get_disabled(val, default):
            return {'base_url': docker_mod.NOTSET,
                    'version': docker_mod.NOTSET,
                    'docker.url': docker_mod.NOTSET,
                    'docker.version': docker_mod.NOTSET,
                    'docker.machine': docker_mod.NOTSET,
                    'docker.update_mine': False}[val]

        def config_get_enabled(val, default):
            return {'base_url': docker_mod.NOTSET,
                    'version': docker_mod.NOTSET,
                    'docker.url': docker_mod.NOTSET,
                    'docker.version': docker_mod.NOTSET,
                    'docker.machine': docker_mod.NOTSET,
                    'docker.update_mine': True}[val]

        mine_mock = Mock()
        dunder_salt = {
            'config.get': MagicMock(side_effect=config_get_disabled),
            'mine.send': mine_mock,
        }
        with patch.dict(docker_mod.__salt__, dunder_salt), \
                patch.dict(docker_mod.__context__, {'docker.client': Mock()}), \
                patch.object(docker_mod, 'state', MagicMock(return_value='stopped')):
            docker_mod.stop('foo', timeout=1)
            mine_mock.assert_not_called()

        with patch.dict(docker_mod.__salt__, dunder_salt), \
                patch.dict(docker_mod.__context__, {'docker.client': Mock()}), \
                patch.object(docker_mod, 'state', MagicMock(return_value='stopped')):
            dunder_salt['config.get'].side_effect = config_get_enabled
            docker_mod.stop('foo', timeout=1)
            self.assert_called_once(mine_mock)

    @skipIf(_docker_py_version() < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_list_networks(self, *args):
        '''
        test list networks.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        client.networks = Mock(return_value=[
            {'Name': 'foo',
             'Id': '01234',
             'Containers': {}}
        ])
        get_client_mock = MagicMock(return_value=client)
        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.networks(names=['foo'], ids=['01234'])
        client.networks.assert_called_once_with(names=['foo'], ids=['01234'])

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_create_network(self, *args):
        '''
        test create network.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.create_network('foo',
                                          driver='bridge',
                                          driver_opts={},
                                          gateway='192.168.0.1',
                                          ip_range='192.168.0.128/25',
                                          subnet='192.168.0.0/24'
                                          )
        client.create_network.assert_called_once_with('foo',
                                                      driver='bridge',
                                                      options={},
                                                      ipam={
                                                          'Config': [{
                                                              'Gateway': '192.168.0.1',
                                                              'IPRange': '192.168.0.128/25',
                                                              'Subnet': '192.168.0.0/24'
                                                          }],
                                                          'Driver': 'default',
                                                      },
                                                      check_duplicate=True)

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_remove_network(self, *args):
        '''
        test remove network.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.remove_network('foo')
        client.remove_network.assert_called_once_with('foo')

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_inspect_network(self, *args):
        '''
        test inspect network.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.inspect_network('foo')
        client.inspect_network.assert_called_once_with('foo')

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_connect_container_to_network(self, *args):
        '''
        test connect_container_to_network
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        context = {'docker.exec_driver': 'docker-exec'}

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.dict(docker_mod.__context__, context):
                with patch.object(docker_mod, '_get_client', get_client_mock):
                    docker_mod.connect_container_to_network('container', 'foo')
        client.connect_container_to_network.assert_called_once_with(
            'container', 'foo')

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_disconnect_container_from_network(self, *args):
        '''
        test disconnect_container_from_network
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        host_config = {}
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.disconnect_container_from_network('container', 'foo')
        client.disconnect_container_from_network.assert_called_once_with(
            'container', 'foo')

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_list_volumes(self, *args):
        '''
        test list volumes.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.volumes(
                    filters={'dangling': [True]},
                )
        client.volumes.assert_called_once_with(
            filters={'dangling': [True]},
        )

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_create_volume(self, *args):
        '''
        test create volume.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.create_volume(
                    'foo',
                    driver='bridge',
                    driver_opts={},
                )
        client.create_volume.assert_called_once_with(
                    'foo',
                    driver='bridge',
                    driver_opts={},
        )

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_remove_volume(self, *args):
        '''
        test remove volume.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.remove_volume('foo')
        client.remove_volume.assert_called_once_with('foo')

    @skipIf(docker_version < (1, 5, 0),
            'docker module must be installed to run this test or is too old. >=1.5.0')
    def test_inspect_volume(self, *args):
        '''
        test inspect volume.
        '''
        __salt__ = {
            'config.get': Mock(),
            'mine.send': Mock(),
        }
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        with patch.dict(docker_mod.__dict__,
                        {'__salt__': __salt__}):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod.inspect_volume('foo')
        client.inspect_volume.assert_called_once_with('foo')

    def test_wait_success(self):
        client = Mock()
        client.api_version = '1.21'
        client.wait = Mock(return_value=0)
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=[
            {'State': {'Running': True}},
            {'State': {'Stopped': True}}])
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo')
        self.assertEqual(result, {'result': True,
                                  'exit_status': 0,
                                  'state': {'new': 'stopped',
                                            'old': 'running'}})

    def test_wait_fails_already_stopped(self):
        client = Mock()
        client.api_version = '1.21'
        client.wait = Mock(return_value=0)
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=[
            {'State': {'Stopped': True}},
            {'State': {'Stopped': True}},
        ])
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo')
        self.assertEqual(result, {'result': False,
                                  'comment': "Container 'foo' already stopped",
                                  'exit_status': 0,
                                  'state': {'new': 'stopped',
                                            'old': 'stopped'}})

    def test_wait_success_already_stopped(self):
        client = Mock()
        client.api_version = '1.21'
        client.wait = Mock(return_value=0)
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=[
            {'State': {'Stopped': True}},
            {'State': {'Stopped': True}},
        ])
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo', ignore_already_stopped=True)
        self.assertEqual(result, {'result': True,
                                  'comment': "Container 'foo' already stopped",
                                  'exit_status': 0,
                                  'state': {'new': 'stopped',
                                            'old': 'stopped'}})

    def test_wait_success_absent_container(self):
        client = Mock()
        client.api_version = '1.21'
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=CommandExecutionError)
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo', ignore_already_stopped=True)
        self.assertEqual(result, {'result': True,
                                  'comment': "Container 'foo' absent"})

    def test_wait_fails_on_exit_status(self):
        client = Mock()
        client.api_version = '1.21'
        client.wait = Mock(return_value=1)
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=[
            {'State': {'Running': True}},
            {'State': {'Stopped': True}}])
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo', fail_on_exit_status=True)
        self.assertEqual(result, {'result': False,
                                  'exit_status': 1,
                                  'state': {'new': 'stopped',
                                            'old': 'running'}})

    def test_wait_fails_on_exit_status_and_already_stopped(self):
        client = Mock()
        client.api_version = '1.21'
        client.wait = Mock(return_value=1)
        get_client_mock = MagicMock(return_value=client)

        docker_inspect_container = Mock(side_effect=[
            {'State': {'Stopped': True}},
            {'State': {'Stopped': True}}])
        with patch.object(docker_mod, 'inspect_container',
                          docker_inspect_container):
            with patch.object(docker_mod, '_get_client', get_client_mock):
                docker_mod._clear_context()
                result = docker_mod.wait('foo',
                                           ignore_already_stopped=True,
                                           fail_on_exit_status=True)
        self.assertEqual(result, {'result': False,
                                  'comment': "Container 'foo' already stopped",
                                  'exit_status': 1,
                                  'state': {'new': 'stopped',
                                            'old': 'stopped'}})

    def test_sls_build(self, *args):
        '''
        test build sls image.
        '''
        docker_start_mock = MagicMock(
            return_value={})
        docker_create_mock = MagicMock(
            return_value={'Id': 'ID', 'Name': 'NAME'})
        docker_stop_mock = MagicMock(
            return_value={'state': {'old': 'running', 'new': 'stopped'},
                          'result': True})
        docker_rm_mock = MagicMock(return_value={})
        docker_commit_mock = MagicMock(
            return_value={'Id': 'ID2', 'Image': 'foo', 'Time_Elapsed': 42})

        docker_sls_mock = MagicMock(
            return_value={
                "file_|-/etc/test.sh_|-/etc/test.sh_|-managed": {
                    "comment": "File /etc/test.sh is in the correct state",
                    "name": "/etc/test.sh",
                    "start_time": "07:04:26.834792",
                    "result": True,
                    "duration": 13.492,
                    "__run_num__": 0,
                    "changes": {}
                },
                "test_|-always-passes_|-foo_|-succeed_without_changes": {
                    "comment": "Success!",
                    "name": "foo",
                    "start_time": "07:04:26.848915",
                    "result": True,
                    "duration": 0.363,
                    "__run_num__": 1,
                    "changes": {}
                }
            })

        ret = None
        with patch.object(docker_mod, 'start_', docker_start_mock), \
                patch.object(docker_mod, 'create', docker_create_mock), \
                patch.object(docker_mod, 'stop', docker_stop_mock), \
                patch.object(docker_mod, 'commit', docker_commit_mock), \
                patch.object(docker_mod, 'sls', docker_sls_mock), \
                patch.object(docker_mod, 'rm_', docker_rm_mock):
            ret = docker_mod.sls_build('foo', mods='foo')
        docker_create_mock.assert_called_once_with(
            cmd='sleep infinity',
            image='opensuse/python', interactive=True, tty=True)
        docker_start_mock.assert_called_once_with('ID')
        docker_sls_mock.assert_called_once_with('ID', 'foo')
        docker_stop_mock.assert_called_once_with('ID')
        docker_rm_mock.assert_called_once_with('ID')
        docker_commit_mock.assert_called_once_with('ID', 'foo', tag='latest')
        self.assertEqual(
            {'Id': 'ID2', 'Image': 'foo', 'Time_Elapsed': 42}, ret)

    def test_sls_build_dryrun(self, *args):
        '''
        test build sls image in dryrun mode.
        '''
        docker_start_mock = MagicMock(
            return_value={})
        docker_create_mock = MagicMock(
            return_value={'Id': 'ID', 'Name': 'NAME'})
        docker_stop_mock = MagicMock(
            return_value={'state': {'old': 'running', 'new': 'stopped'},
                          'result': True})
        docker_rm_mock = MagicMock(
            return_value={})

        docker_sls_mock = MagicMock(
            return_value={
                "file_|-/etc/test.sh_|-/etc/test.sh_|-managed": {
                    "comment": "File /etc/test.sh is in the correct state",
                    "name": "/etc/test.sh",
                    "start_time": "07:04:26.834792",
                    "result": True,
                    "duration": 13.492,
                    "__run_num__": 0,
                    "changes": {}
                },
                "test_|-always-passes_|-foo_|-succeed_without_changes": {
                    "comment": "Success!",
                    "name": "foo",
                    "start_time": "07:04:26.848915",
                    "result": True,
                    "duration": 0.363,
                    "__run_num__": 1,
                    "changes": {}
                }
            })

        ret = None
        with patch.object(docker_mod, 'start_', docker_start_mock), \
                patch.object(docker_mod, 'create', docker_create_mock), \
                patch.object(docker_mod, 'stop', docker_stop_mock), \
                patch.object(docker_mod, 'rm_', docker_rm_mock), \
                patch.object(docker_mod, 'sls', docker_sls_mock):
            ret = docker_mod.sls_build('foo', mods='foo', dryrun=True)
        docker_create_mock.assert_called_once_with(
            cmd='sleep infinity',
            image='opensuse/python', interactive=True, tty=True)
        docker_start_mock.assert_called_once_with('ID')
        docker_sls_mock.assert_called_once_with('ID', 'foo')
        docker_stop_mock.assert_called_once_with('ID')
        docker_rm_mock.assert_called_once_with('ID')
        self.assertEqual(
                {
                "file_|-/etc/test.sh_|-/etc/test.sh_|-managed": {
                    "comment": "File /etc/test.sh is in the correct state",
                    "name": "/etc/test.sh",
                    "start_time": "07:04:26.834792",
                    "result": True,
                    "duration": 13.492,
                    "__run_num__": 0,
                    "changes": {}
                },
                "test_|-always-passes_|-foo_|-succeed_without_changes": {
                    "comment": "Success!",
                    "name": "foo",
                    "start_time": "07:04:26.848915",
                    "result": True,
                    "duration": 0.363,
                    "__run_num__": 1,
                    "changes": {}
                }
                },
                ret)

    def test_call_success(self):
        '''
        test module calling inside containers
        '''
        ret = None
        docker_run_all_mock = MagicMock(
            return_value={
                'retcode': 0,
                'stdout': '{"retcode": 0, "comment": "container cmd"}',
                'stderr': 'err',
            })
        docker_copy_to_mock = MagicMock(
            return_value={
                'retcode': 0
            })
        docker_config_mock = MagicMock(
            return_value=''
            )
        client = Mock()
        client.put_archive = Mock()
        get_client_mock = MagicMock(return_value=client)

        context = {'docker.exec_driver': 'docker-exec'}
        salt_dunder = {'config.option': docker_config_mock}

        with patch.object(docker_mod, 'run_all', docker_run_all_mock), \
                patch.object(docker_mod, 'copy_to', docker_copy_to_mock), \
                patch.object(docker_mod, '_get_client', get_client_mock), \
                patch.dict(docker_mod.__opts__, {'cachedir': '/tmp'}), \
                patch.dict(docker_mod.__salt__, salt_dunder), \
                patch.dict(docker_mod.__context__, context):
            # call twice to verify tmp path later
            for i in range(2):
                ret = docker_mod.call('ID', 'test.arg', 1, 2, arg1='val1')

        # Check that the directory is different each time
        # [ call(name, [args]), ...
        self.maxDiff = None
        self.assertIn('mkdir', docker_run_all_mock.mock_calls[0][1][1])
        self.assertIn('mkdir', docker_run_all_mock.mock_calls[4][1][1])
        self.assertNotEqual(docker_run_all_mock.mock_calls[0][1][1],
                            docker_run_all_mock.mock_calls[4][1][1])

        self.assertIn('salt-call', docker_run_all_mock.mock_calls[2][1][1])
        self.assertIn('salt-call', docker_run_all_mock.mock_calls[6][1][1])
        self.assertNotEqual(docker_run_all_mock.mock_calls[2][1][1],
                            docker_run_all_mock.mock_calls[6][1][1])

        # check thin untar
        self.assertIn('tarfile', docker_run_all_mock.mock_calls[1][1][1])
        self.assertIn('tarfile', docker_run_all_mock.mock_calls[5][1][1])
        self.assertNotEqual(docker_run_all_mock.mock_calls[1][1][1],
                            docker_run_all_mock.mock_calls[5][1][1])

        # check directory cleanup
        self.assertIn('rm -rf', docker_run_all_mock.mock_calls[3][1][1])
        self.assertIn('rm -rf', docker_run_all_mock.mock_calls[7][1][1])
        self.assertNotEqual(docker_run_all_mock.mock_calls[3][1][1],
                            docker_run_all_mock.mock_calls[7][1][1])

        self.assertEqual({"retcode": 0, "comment": "container cmd"}, ret)

    def test_images_with_empty_tags(self):
        '''
        docker 1.12 reports also images without tags with `null`.
        '''
        client = Mock()
        client.api_version = '1.24'
        client.images = Mock(
            return_value=[{'Id': 'sha256:abcde',
                           'RepoTags': None},
                          {'Id': 'sha256:abcdef'},
                          {'Id': 'sha256:abcdefg',
                           'RepoTags': ['image:latest']}])
        get_client_mock = MagicMock(return_value=client)

        with patch.object(docker_mod, '_get_client', get_client_mock):
            docker_mod._clear_context()
            result = docker_mod.images()
        self.assertEqual(result,
                         {'sha256:abcdefg': {'RepoTags': ['image:latest']}})

    def test_compare_container_image_id_resolution(self):
        '''
        Test comparing two containers when one's inspect output is an ID and
        not formatted in image:tag notation.
        '''
        def _inspect_container_effect(id_):
            return {
                'container1': {'Config': {'Image': 'realimage:latest'},
                               'HostConfig': {}},
                'container2': {'Config': {'Image': 'image_id'},
                               'HostConfig': {}},
            }[id_]

        def _inspect_image_effect(id_):
            return {
                'realimage:latest': {'Id': 'image_id'},
                'image_id': {'Id': 'image_id'},
            }[id_]

        inspect_container_mock = MagicMock(side_effect=_inspect_container_effect)
        inspect_image_mock = MagicMock(side_effect=_inspect_image_effect)

        with patch.object(docker_mod, 'inspect_container', inspect_container_mock):
            with patch.object(docker_mod, 'inspect_image', inspect_image_mock):
                ret = docker_mod.compare_containers('container1', 'container2')
                self.assertEqual(ret, {})

    def test_compare_container_ulimits_order(self):
        '''
        Test comparing two containers when the order of the Ulimits HostConfig
        values are different, but the values are the same.
        '''
        def _inspect_container_effect(id_):
            return {
                'container1': {'Config': {},
                               'HostConfig': {
                                   'Ulimits': [
                                       {'Hard': -1, 'Soft': -1, 'Name': 'core'},
                                       {'Hard': 65536, 'Soft': 65536, 'Name': 'nofile'}
                                   ]
                               }},
                'container2': {'Config': {},
                               'HostConfig': {
                                   'Ulimits': [
                                       {'Hard': 65536, 'Soft': 65536, 'Name': 'nofile'},
                                       {'Hard': -1, 'Soft': -1, 'Name': 'core'}
                                   ]
                               }},
            }[id_]

        inspect_container_mock = MagicMock(side_effect=_inspect_container_effect)

        with patch.object(docker_mod, 'inspect_container', inspect_container_mock):
            ret = docker_mod.compare_container('container1', 'container2')
            self.assertEqual(ret, {})

    def test_compare_container_env_order(self):
        '''
        Test comparing two containers when the order of the Env HostConfig
        values are different, but the values are the same.
        '''
        def _inspect_container_effect(id_):
            return {
                'container1': {'Config': {},
                               'HostConfig': {
                                   'Env': [
                                       'FOO=bar',
                                       'HELLO=world',
                                   ]
                               }},
                'container2': {'Config': {},
                               'HostConfig': {
                                   'Env': [
                                       'HELLO=world',
                                       'FOO=bar',
                                   ]
                               }},
            }[id_]

        inspect_container_mock = MagicMock(side_effect=_inspect_container_effect)

        with patch.object(docker_mod, 'inspect_container', inspect_container_mock):
            ret = docker_mod.compare_container('container1', 'container2')
            self.assertEqual(ret, {})

    def test_resolve_tag(self):
        '''
        Test the resolve_tag function. It runs docker.insect_image on the image
        name passed and then looks for the RepoTags key in the result
        '''
        id_ = 'sha256:6ad733544a6317992a6fac4eb19fe1df577d4dec7529efec28a5bd0edad0fd30'
        tags = ['foo:latest', 'foo:bar']
        mock_tagged = MagicMock(return_value={'Id': id_, 'RepoTags': tags})
        mock_untagged = MagicMock(return_value={'Id': id_, 'RepoTags': []})
        mock_unexpected = MagicMock(return_value={'Id': id_})
        mock_not_found = MagicMock(side_effect=CommandExecutionError())

        with patch.object(docker_mod, 'inspect_image', mock_tagged):
            self.assertEqual(docker_mod.resolve_tag('foo'), tags[0])
            self.assertEqual(docker_mod.resolve_tag('foo', all=True), tags)

        with patch.object(docker_mod, 'inspect_image', mock_untagged):
            self.assertEqual(docker_mod.resolve_tag('foo'), id_)
            self.assertEqual(docker_mod.resolve_tag('foo', all=True), [id_])

        with patch.object(docker_mod, 'inspect_image', mock_unexpected):
            self.assertEqual(docker_mod.resolve_tag('foo'), id_)
            self.assertEqual(docker_mod.resolve_tag('foo', all=True), [id_])

        with patch.object(docker_mod, 'inspect_image', mock_not_found):
            self.assertIs(docker_mod.resolve_tag('foo'), False)
            self.assertIs(docker_mod.resolve_tag('foo', all=True), False)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Libs
import salt.utils.platform
import salt.modules.status as status
from salt.exceptions import CommandExecutionError
from salt.ext import six

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch,
    mock_open,
)


class StatusTestCase(TestCase, LoaderModuleMockMixin):
    '''
    test modules.status functions
    '''
    def setup_loader_modules(self):
        return {status: {}}

    def _set_up_test_uptime(self):
        '''
        Define common mock data for status.uptime tests
        '''
        class MockData(object):
            '''
            Store mock data
            '''

        m = MockData()
        m.now = 1477004312
        m.ut = 1540154.00
        m.idle = 3047777.32
        m.ret = {
            'users': 3,
            'seconds': 1540154,
            'since_t': 1475464158,
            'days': 17,
            'since_iso': '2016-10-03T03:09:18',
            'time': '19:49',
        }

        return m

    def _set_up_test_uptime_sunos(self):
        '''
        Define common mock data for cmd.run_all for status.uptime on SunOS
        '''
        class MockData(object):
            '''
            Store mock data
            '''

        m = MockData()
        m.ret = {
            'retcode': 0,
            'stdout': 'unix:0:system_misc:boot_time    1475464158',
        }

        return m

    def test_uptime_linux(self):
        '''
        Test modules.status.uptime function for Linux
        '''
        m = self._set_up_test_uptime()

        with patch.multiple(salt.utils.platform,
                            is_linux=MagicMock(return_value=True),
                            is_sunos=MagicMock(return_value=False),
                            is_darwin=MagicMock(return_value=False),
                            is_freebsd=MagicMock(return_value=False),
                            is_openbsd=MagicMock(return_value=False),
                            is_netbsd=MagicMock(return_value=False)), \
                patch('salt.utils.path.which', MagicMock(return_value=True)), \
                patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=os.linesep.join(['1', '2', '3']))}), \
                patch('time.time', MagicMock(return_value=m.now)), \
                patch('os.path.exists', MagicMock(return_value=True)):
            proc_uptime = salt.utils.stringutils.to_str('{0} {1}'.format(m.ut, m.idle))

            with patch('salt.utils.files.fopen', mock_open(read_data=proc_uptime)):
                ret = status.uptime()
                self.assertDictEqual(ret, m.ret)
            with patch('os.path.exists', MagicMock(return_value=False)):
                with self.assertRaises(CommandExecutionError):
                    status.uptime()

    def test_uptime_sunos(self):
        '''
        Test modules.status.uptime function for SunOS
        '''
        m = self._set_up_test_uptime()
        m2 = self._set_up_test_uptime_sunos()
        with patch.multiple(salt.utils.platform,
                            is_linux=MagicMock(return_value=False),
                            is_sunos=MagicMock(return_value=True),
                            is_darwin=MagicMock(return_value=False),
                            is_freebsd=MagicMock(return_value=False),
                            is_openbsd=MagicMock(return_value=False),
                            is_netbsd=MagicMock(return_value=False)), \
                patch('salt.utils.path.which', MagicMock(return_value=True)), \
                patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=os.linesep.join(['1', '2', '3'])),
                                             'cmd.run_all': MagicMock(return_value=m2.ret)}), \
                patch('time.time', MagicMock(return_value=m.now)):
            ret = status.uptime()
            self.assertDictEqual(ret, m.ret)

    def test_uptime_macos(self):
        '''
        Test modules.status.uptime function for macOS
        '''
        m = self._set_up_test_uptime()

        kern_boottime = ('{{ sec = {0}, usec = {1:0<6} }} Mon Oct 03 03:09:18.23 2016'
                         ''.format(*six.text_type(m.now - m.ut).split('.')))
        with patch.multiple(salt.utils.platform,
                            is_linux=MagicMock(return_value=False),
                            is_sunos=MagicMock(return_value=False),
                            is_darwin=MagicMock(return_value=True),
                            is_freebsd=MagicMock(return_value=False),
                            is_openbsd=MagicMock(return_value=False),
                            is_netbsd=MagicMock(return_value=False)), \
                patch('salt.utils.path.which', MagicMock(return_value=True)), \
                patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=os.linesep.join(['1', '2', '3'])),
                                             'sysctl.get': MagicMock(return_value=kern_boottime)}), \
                patch('time.time', MagicMock(return_value=m.now)):

            ret = status.uptime()
            self.assertDictEqual(ret, m.ret)

            with patch.dict(status.__salt__, {'sysctl.get': MagicMock(return_value='')}):
                with self.assertRaises(CommandExecutionError):
                    status.uptime()

    def test_uptime_return_success_not_supported(self):
        '''
        Test modules.status.uptime function for other platforms
        '''
        with patch.multiple(salt.utils.platform,
                            is_linux=MagicMock(return_value=False),
                            is_sunos=MagicMock(return_value=False),
                            is_darwin=MagicMock(return_value=False),
                            is_freebsd=MagicMock(return_value=False),
                            is_openbsd=MagicMock(return_value=False),
                            is_netbsd=MagicMock(return_value=False)):
            exc_mock = MagicMock(side_effect=CommandExecutionError)
            with self.assertRaises(CommandExecutionError):
                with patch.dict(status.__salt__, {'cmd.run': exc_mock}):
                    status.uptime()

    def _set_up_test_cpustats_openbsd(self):
        '''
        Define mock data for status.cpustats on OpenBSD
        '''
        class MockData(object):
            '''
            Store mock data
            '''

        m = MockData()
        m.ret = {
           '0': {
                'User': '0.0%',
                'Nice': '0.0%',
                'System': '4.5%',
                'Interrupt': '0.5%',
                'Idle': '95.0%',
            }
        }

        return m

    def test_cpustats_openbsd(self):
        '''
        Test modules.status.cpustats function for OpenBSD
        '''
        m = self._set_up_test_cpustats_openbsd()

        systat = '\n' \
                 '\n' \
                 '   1 users Load 0.20 0.07 0.05                        salt.localdomain 09:42:42\n' \
                 'CPU                User           Nice        System     Interrupt          Idle\n' \
                 '0                  0.0%           0.0%          4.5%          0.5%         95.0%\n'

        with patch.multiple(salt.utils.platform,
                            is_linux=MagicMock(return_value=False),
                            is_sunos=MagicMock(return_value=False),
                            is_darwin=MagicMock(return_value=False),
                            is_freebsd=MagicMock(return_value=False),
                            is_openbsd=MagicMock(return_value=True),
                            is_netbsd=MagicMock(return_value=False)), \
                    patch('salt.utils.path.which', MagicMock(return_value=True)), \
                    patch.dict(status.__grains__, {'kernel': 'OpenBSD'}), \
                    patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=systat)}):
            ret = status.cpustats()
            self.assertDictEqual(ret, m.ret)

    def _set_up_test_cpuinfo_bsd(self):
        class MockData(object):
            '''
            Store mock data
            '''

        m = MockData()
        m.ret = {
          'hw.model': 'Intel(R) Core(TM) i5-7287U CPU @ 3.30GHz',
          'hw.ncpu': '4',
        }

        return m

    def test_cpuinfo_freebsd(self):
        m = self._set_up_test_cpuinfo_bsd()
        sysctl = 'hw.model:Intel(R) Core(TM) i5-7287U CPU @ 3.30GHz\nhw.ncpu:4'

        with patch.dict(status.__grains__, {'kernel': 'FreeBSD'}):
            with patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=sysctl)}):
                ret = status.cpuinfo()
                self.assertDictEqual(ret, m.ret)

    def test_cpuinfo_openbsd(self):
        m = self._set_up_test_cpuinfo_bsd()
        sysctl = 'hw.model=Intel(R) Core(TM) i5-7287U CPU @ 3.30GHz\nhw.ncpu=4'

        for bsd in ['NetBSD', 'OpenBSD']:
            with patch.dict(status.__grains__, {'kernel': bsd}):
                with patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=sysctl)}):
                    ret = status.cpuinfo()
                    self.assertDictEqual(ret, m.ret)

    def _set_up_test_meminfo_openbsd(self):
        class MockData(object):
            '''
            Store mock data
            '''

        m = MockData()
        m.ret = {
            'active virtual pages': '355M',
            'free list size': '305M',
            'page faults': '845',
            'pages reclaimed': '1',
            'pages paged in': '2',
            'pages paged out': '3',
            'pages freed': '4',
            'pages scanned': '5'
        }

        return m

    def test_meminfo_openbsd(self):
        m = self._set_up_test_meminfo_openbsd()
        vmstat = ' procs    memory       page                    disks    traps          cpu\n' \
                 ' r   s   avm     fre  flt  re  pi  po  fr  sr cd0 sd0  int   sys   cs us sy id\n' \
                 ' 2 103  355M    305M  845   1   2   3   4   5   0   1   21   682   86  1  1 98'

        with patch.dict(status.__grains__, {'kernel': 'OpenBSD'}):
            with patch.dict(status.__salt__, {'cmd.run': MagicMock(return_value=vmstat)}):
                ret = status.meminfo()
                self.assertDictEqual(ret, m.ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.unit import TestCase

# Import Salt libs
import salt.utils.dicttrim as dicttrimmer

import logging
log = logging.getLogger(__name__)


class DictTrimTestCase(TestCase):

    def setUp(self):
        self.old_dict = {'a': 'b', 'c': 'x' * 10000}
        self.new_dict = {'a': 'b', 'c': 'VALUE_TRIMMED'}

    def test_trim_dict(self):
        ret = dicttrimmer.trim_dict(self.old_dict, 1000)
        self.assertEqual(ret, self.new_dict)


class RecursiveDictTrimTestCase(TestCase):

    def setUp(self):
        self.old_dict = {'a': {'b': 1, 'c': 2, 'e': 'x' * 10000, 'f': '3'}}
        self.new_dict = {'a': {'b': 1, 'c': 2, 'e': 'VALUE_TRIMMED', 'f': '3'}}

    def test_trim_dict(self):
        ret = dicttrimmer.trim_dict(self.old_dict, 1000)
        self.assertEqual(ret, self.new_dict)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Eric Radman <ericshane@eradman.com>
'''

# Import Salt Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.cloud.clouds import joyent


# Stubs
def fake_wait_for_ip(check_for_ip_fn,
                     interval=None,
                     timeout=None,
                     interval_multiplier=None):
    '''
    Callback that returns immediately instead of waiting
    '''
    assert isinstance(interval, int)
    assert isinstance(timeout, int)
    assert isinstance(interval_multiplier, int)
    return check_for_ip_fn()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JoyentTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Unit TestCase for the salt.cloud.clouds.joyent module
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.utils.cloud.wait_for_ip', fake_wait_for_ip)
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            joyent: {
                '__utils__': {
                    'cloud.fire_event': MagicMock(),
                    'cloud.bootstrap': MagicMock()
                },
                '__opts__': {
                    'sock_dir': True,
                    'transport': True,
                    'providers': {'my_joyent': {}},
                    'profiles': {'my_joyent': {}}
                },
                '__active_provider_name__': 'my_joyent:joyent'
            }
        }

    def setUp(self):
        self.vm_ = {
            'profile': 'my_joyent',
            'name': 'vm3',
            'driver': 'joyent',
            'size': 'k4-highcpu-kvm-750M',
            'image': 'freebsd10',
            'location': 'us-east-1'
        }

    def tearDown(self):
        del self.vm_

    def test_query_instance_init(self):
        '''
        Initial provisioning, no IP assigned
        '''
        # Not yet reachable
        reply = (200, {'state': 'provisioning'})
        with patch.object(joyent, 'show_instance', return_value=reply):
            result = joyent.query_instance(self.vm_)
        self.assertTrue(joyent.__utils__['cloud.fire_event'].called_once())
        self.assertEqual(result, None)

    def test_query_instance_has_ip(self):
        '''
        IP address assigned but not yet ready
        '''
        reply = (200, {'primaryIp': '1.1.1.1', 'state': 'provisioning'})
        with patch.object(joyent, 'show_instance', return_value=reply):
            result = joyent.query_instance(self.vm_)
        self.assertTrue(joyent.__utils__['cloud.fire_event'].called_once())
        self.assertEqual(result, None)

    def test_query_instance_ready(self):
        '''
        IP address assigned, and VM is ready
        '''
        reply = (200, {'primaryIp': '1.1.1.1', 'state': 'running'})
        with patch.object(joyent, 'show_instance', return_value=reply):
            result = joyent.query_instance(self.vm_)
        self.assertTrue(joyent.__utils__['cloud.fire_event'].called_once())
        self.assertEqual(result, '1.1.1.1')



# coding: utf-8

# Python libs
from __future__ import absolute_import
import datetime
from uuid import UUID

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, Mock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.journald as journald
import salt.utils.data

import logging
log = logging.getLogger(__name__)

_STUB_JOURNALD_ENTRY = {'_BOOT_ID': UUID('ad3915a5-9008-4fec-a635-140606525497'),
                        '__MONOTONIC_TIMESTAMP': (datetime.timedelta(4, 28586, 703069),
                                                  UUID('ad3915a5-9008-4fec-a635-140606525497')),
                        '_AUDIT_LOGINUID': 1000, 'SYSLOG_FACILITY': 10,
                        '_SYSTEMD_SLICE': u'system.slice',
                        '_GID': 0,
                        '__REALTIME_TIMESTAMP': datetime.datetime(2017, 6, 27,
                                                                  20, 8, 16,
                                                                  468468),
                        '_AUDIT_SESSION': 351, 'PRIORITY': 6,
                        '_TRANSPORT': u'syslog',
                        '_HOSTNAME': u'hostname',
                        '_CAP_EFFECTIVE': u'3fffffffff',
                        '_SYSTEMD_UNIT': u'ssh.service',
                        '_MACHINE_ID': UUID('14fab5bb-228d-414b-bdf4-cbc62cb7ba54'),
                        '_PID': 15091,
                        'SYSLOG_IDENTIFIER': u'sshd',
                        '_SOURCE_REALTIME_TIMESTAMP': datetime.datetime(2017,
                                                                        6,
                                                                        27,
                                                                        20,
                                                                        8,
                                                                        16,
                                                                        468454),
                        '_SYSTEMD_CGROUP': u'/system.slice/ssh.service',
                        '__CURSOR': 's=7711ee01b03446309383870171dd5839;i=a74e;b=ad3915a590084feca635140606525497;m=571f43f8 dd;t=552fc7ed1cdf4;x=4ca0a3d4f1905736',
                        '_COMM': u'sshd',
                        '_CMDLINE': u'sshd: gareth [priv]',
                        '_SYSTEMD_INVOCATION_ID': u'38a5d5aad292426d93bfaab72a69c2ab',
                        '_EXE': u'/usr/sbin/sshd',
                        '_UID': 0,
                        'SYSLOG_PID': 15091,
                        'MESSAGE': u'pam_unix(sshd:session): session opened for user username by (uid=0)'}


class SystemdJournaldMock(Mock):
    ''' Request Mock'''

    returned_once = False

    def get_next(self, *args, **kwargs):
        if not self.returned_once:
            self.returned_once = True
            return _STUB_JOURNALD_ENTRY
        else:
            return None

    def seek_tail(self, *args, **kwargs):
        return {}

    def get_previous(self, *args, **kwargs):
        return {}


SYSTEMD_MOCK = SystemdJournaldMock()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class JournaldBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.journald
    '''

    def setup_loader_modules(self):
        return {
            journald: {
                '__context__': {
                    'systemd.journald': SYSTEMD_MOCK,
                },
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = journald.validate(config)

        self.assertEqual(ret, (False, 'Configuration for journald beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = journald.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

    def test_journald_match(self):
        config = [{'services': {'sshd': {'SYSLOG_IDENTIFIER': 'sshd',
                                         'PRIORITY': 6}}}]

        ret = journald.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

        _expected_return = salt.utils.data.simple_types_filter(_STUB_JOURNALD_ENTRY)
        _expected_return['tag'] = 'sshd'

        ret = journald.beacon(config)
        self.assertEqual(ret, [_expected_return])



# -*- coding: utf-8 -*-
'''
This is a module used in unit.utils.cache to test the context wrapper functions
'''

import salt.utils.cache


def __virtual__():
    return True


@salt.utils.cache.context_cache
def test_context_module():
    if 'called' in __context__:
        __context__['called'] += 1
    else:
        __context__['called'] = 0
    return __context__



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Shane Lee <slee@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    patch,
    mock_open,
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.grains.fibre_channel as fibre_channel


@skipIf(NO_MOCK, NO_MOCK_REASON)
class FibreChannelGrainsTestCase(TestCase):
    '''
    Test cases for iscsi grains
    '''
    def test_windows_fibre_channel_wwns_grains(self):
        wwns = ['20:00:00:25:b5:11:11:4c',
                '20:00:00:25:b5:11:11:5c',
                '20:00:00:25:b5:44:44:4c',
                '20:00:00:25:b5:44:44:5c']
        cmd_run_mock = MagicMock(return_value=wwns)
        with patch('salt.modules.cmdmod.powershell', cmd_run_mock):
            ret = fibre_channel._windows_wwns()
        assert ret == wwns, ret

    def test_linux_fibre_channel_wwns_grains(self):

        contents = ['0x500143802426baf4', '0x500143802426baf5']
        files = ['file1', 'file2']
        with patch('glob.glob', MagicMock(return_value=files)), \
                patch('salt.utils.files.fopen', mock_open(read_data=contents)):
            ret = fibre_channel._linux_wwns()

        assert ret == ['500143802426baf4', '500143802426baf5'], ret



# -*- coding: utf-8 -*-
'''
    :codeauthor: Bo Maryniuk <bo@suse.de>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import TestCase
from tests.support.mock import MagicMock, patch
from tests.unit.cloud.clouds import _preferred_ip

# Import Salt libs
from salt.cloud.clouds import nova


class NovaTestCase(TestCase):
    '''
    Test case for openstack
    '''
    PRIVATE_IPS = ['0.0.0.0', '1.1.1.1', '2.2.2.2']

    @patch('salt.cloud.clouds.nova.show_instance',
           MagicMock(return_value={'state': 'ACTIVE',
                                   'public_ips': [],
                                   'addresses': [],
                                   'private_ips': PRIVATE_IPS}))
    @patch('salt.cloud.clouds.nova.rackconnect', MagicMock(return_value=False))
    @patch('salt.cloud.clouds.nova.rackconnectv3', MagicMock(return_value={'mynet': ['1.1.1.1']}))
    @patch('salt.cloud.clouds.nova.cloudnetwork', MagicMock(return_value=False))
    @patch('salt.cloud.clouds.nova.managedcloud', MagicMock(return_value=False))
    @patch('salt.cloud.clouds.nova.preferred_ip', _preferred_ip(PRIVATE_IPS, ['0.0.0.0']))
    @patch('salt.cloud.clouds.nova.ssh_interface', MagicMock(return_value='public_ips'))
    def test_query_node_data_filter_preferred_ip_addresses(self):
        '''
        Test if query node data is filtering out unpreferred IP addresses.
        '''
        vm = {'name': None}
        data = MagicMock()
        data.public_ips = []

        assert nova._query_node_data(vm, data, MagicMock()).public_ips == ['0.0.0.0']



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.json
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import textwrap

# Import Salt Testing libs
from tests.support.helpers import with_tempfile
from tests.support.mock import patch, MagicMock, NO_MOCK, NO_MOCK_REASON
from tests.support.unit import TestCase, LOREM_IPSUM, skipIf

# Import Salt libs
import salt.utils.files
import salt.utils.json
import salt.utils.platform
import salt.utils.stringutils
from salt.ext import six


class JSONTestCase(TestCase):
    data = {
        'спам': 'яйца',
        'list': [1, 2, 'three'],
        'dict': {'subdict': {'спам': 'яйца'}},
        'True': False,
        'float': 1.5,
        'None': None,
    }

    serialized = salt.utils.stringutils.to_str(
        '{"None": null, "True": false, "dict": {"subdict": {"спам": "яйца"}}, "float": 1.5, "list": [1, 2, "three"], "спам": "яйца"}'
    )

    serialized_indent4 = salt.utils.stringutils.to_str(textwrap.dedent('''\
        {
            "None": null,
            "True": false,
            "dict": {
                "subdict": {
                    "спам": "яйца"
                }
            },
            "float": 1.5,
            "list": [
                1,
                2,
                "three"
            ],
            "спам": "яйца"
        }'''))

    def test_find_json(self):
        test_sample_json = '''
                            {
                                "glossary": {
                                    "title": "example glossary",
                                    "GlossDiv": {
                                        "title": "S",
                                        "GlossList": {
                                            "GlossEntry": {
                                                "ID": "SGML",
                                                "SortAs": "SGML",
                                                "GlossTerm": "Standard Generalized Markup Language",
                                                "Acronym": "SGML",
                                                "Abbrev": "ISO 8879:1986",
                                                "GlossDef": {
                                                    "para": "A meta-markup language, used to create markup languages such as DocBook.",
                                                    "GlossSeeAlso": ["GML", "XML"]
                                                },
                                                "GlossSee": "markup"
                                            }
                                        }
                                    }
                                }
                            }
                            '''
        expected_ret = {'glossary': {'GlossDiv': {'GlossList': {'GlossEntry': {
            'GlossDef': {'GlossSeeAlso': ['GML', 'XML'],
                         'para': 'A meta-markup language, used to create markup languages such as DocBook.'},
            'GlossSee': 'markup', 'Acronym': 'SGML', 'GlossTerm': 'Standard Generalized Markup Language',
            'SortAs': 'SGML',
            'Abbrev': 'ISO 8879:1986', 'ID': 'SGML'}}, 'title': 'S'}, 'title': 'example glossary'}}

        # First test the valid JSON
        ret = salt.utils.json.find_json(test_sample_json)
        self.assertDictEqual(ret, expected_ret)

        # Now pre-pend some garbage and re-test
        garbage_prepend_json = '{0}{1}'.format(LOREM_IPSUM, test_sample_json)
        ret = salt.utils.json.find_json(garbage_prepend_json)
        self.assertDictEqual(ret, expected_ret)

        # Test to see if a ValueError is raised if no JSON is passed in
        self.assertRaises(ValueError, salt.utils.json.find_json, LOREM_IPSUM)

    @skipIf(salt.utils.platform.is_windows(), 'skip until we figure out what to do about decoding unicode on windows')
    @skipIf(not six.PY2, 'Test only needed on Python 2')
    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_find_json_unicode_splitlines(self):
        '''
        Tests a case in salt-ssh where a unicode string is split into a list of
        str types by .splitlines().
        '''
        raw = '{"foo": "öäü"}'
        mock_split = MagicMock(return_value=[raw.encode('utf8')])

        with patch.object(salt.utils.json, '__split', mock_split):
            ret = salt.utils.json.find_json(raw)
            self.assertEqual(ret, {'foo': 'öäü'})

    def test_dumps_loads(self):
        '''
        Test dumping to and loading from a string
        '''
        # Dump with no indentation
        ret = salt.utils.json.dumps(self.data, sort_keys=True)
        # Make sure the result is as expected
        self.assertEqual(ret, self.serialized)
        # Loading it should be equal to the original data
        self.assertEqual(salt.utils.json.loads(ret), self.data)

        # Dump with 4 spaces of indentation
        ret = salt.utils.json.dumps(self.data, sort_keys=True, indent=4)
        # Make sure the result is as expected. Note that in Python 2, dumping
        # results in trailing whitespace on lines ending in a comma. So, for a
        # proper comparison, we will have to run rstrip on each line of the
        # return and then stitch it back together.
        ret = str('\n').join([x.rstrip() for x in ret.splitlines()])  # future lint: disable=blacklisted-function
        self.assertEqual(ret, self.serialized_indent4)
        # Loading it should be equal to the original data
        self.assertEqual(salt.utils.json.loads(ret), self.data)

    @with_tempfile()
    def test_dump_load(self, json_out):
        '''
        Test dumping to and loading from a file handle
        '''
        with salt.utils.files.fopen(json_out, 'wb') as fp_:
            fp_.write(salt.utils.stringutils.to_bytes(salt.utils.json.dumps(self.data)))
        with salt.utils.files.fopen(json_out, 'rb') as fp_:
            ret = salt.utils.json.loads(salt.utils.stringutils.to_unicode(fp_.read()))
            # Loading should be equal to the original data
            self.assertEqual(ret, self.data)



# -*- coding: utf-8 -*-
'''
Validate the boto_iam module
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

# Import 3rd-party libs
try:
    import boto
    NO_BOTO_MODULE = False
except ImportError:
    NO_BOTO_MODULE = True


@skipIf(
    NO_BOTO_MODULE,
    'Please install the boto library before running boto integration tests.'
)
class BotoIAMTest(ModuleCase):

    def setUp(self):
        try:
            boto.connect_iam()
        except boto.exception.NoAuthHandlerFound:
            self.skipTest('Please setup boto AWS credentials before running boto integration tests.')

    def test_get_account_id(self):
        ret = self.run_function('boto_iam.get_account_id')
        # The AWS account ID is a 12-digit number.
        # http://docs.aws.amazon.com/general/latest/gr/acct-identifiers.html
        self.assertRegex(ret, r'^\d{12}$')



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, Mock, MagicMock, patch

# Import salt libs
import salt.modules.postgres as postgresmod
import salt.states.postgres_database as postgres_database
import salt.states.postgres_user as postgres_user
import salt.states.postgres_group as postgres_group
import salt.states.postgres_extension as postgres_extension
import salt.states.postgres_schema as postgres_schema


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresUserTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            'config.option': Mock(),
            'cmd.run_all': Mock(),
            'file.chown': Mock(),
            'file.remove': Mock(),
        }
        self.addCleanup(delattr, self, 'salt_stub')
        return {
            postgres_database: {},
            postgres_group: {},
            postgres_extension: {},
            postgres_schema: {},
            postgres_user: {
                '__grains__': {'os_family': 'Linux'},
                '__salt__': self.salt_stub,
                '__opts__': {'test': False},
            }
        }

    def test_present__creation(self):
        # test=True
        with patch.dict(postgres_user.__salt__, {'postgres.role_get': Mock(return_value=None),
                                                 'postgres.user_create': MagicMock()}):
            with patch.dict(postgres_user.__opts__, {'test': True}):
                ret = postgres_user.present('foo')
                self.assertEqual(
                    ret,
                    {'comment': 'User foo is set to be created',
                     'changes': {}, 'name': 'foo', 'result': None}
                )
                self.assertEqual(self.salt_stub['postgres.user_create'].call_count, 0)

            # test=False
            ret = postgres_user.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'The user foo has been created',
                 'changes': {'foo': 'Present'}, 'name': 'foo', 'result': True}
            )
            self.salt_stub['postgres.user_create'].assert_called_once_with(username='foo',
                                                                           superuser=None,
                                                                           encrypted=True,
                                                                           runas=None,
                                                                           inherit=None,
                                                                           rolepassword=None,
                                                                           port=None,
                                                                           replication=None,
                                                                           host=None,
                                                                           createroles=None,
                                                                           user=None,
                                                                           groups=None,
                                                                           maintenance_db=None,
                                                                           login=None,
                                                                           password=None,
                                                                           valid_until=None,
                                                                           createdb=None)

    def test_present__update(self):
        # test=True
        with patch.dict(postgres_user.__salt__, {'postgres.role_get': Mock(return_value={
                                                     'can create databases': False,
                                                     'can create roles': False,
                                                     'can login': False,
                                                     'can update system catalogs': False,
                                                     'connections': None,
                                                     'defaults variables': {},
                                                     'expiry time': None,
                                                     'inherits privileges': True,
                                                     'replication': False,
                                                     'superuser': False,
                                                 }),
                                                 'postgres.user_update': MagicMock()}):
            with patch.dict(postgres_user.__opts__, {'test': True}):
                ret = postgres_user.present('foo', login=True, replication=False)
                self.assertEqual(
                    ret,
                    {'comment': 'User foo is set to be updated',
                     'changes': {'foo': {'login': True}}, 'name': 'foo', 'result': None}
                )
                self.assertEqual(self.salt_stub['postgres.user_update'].call_count, 0)

            # test=False
            ret = postgres_user.present('foo', login=True, replication=False)
            self.assertEqual(
                ret,
                {'comment': 'The user foo has been updated',
                 'changes': {'foo': {'login': True}}, 'name': 'foo', 'result': True}
            )
            self.salt_stub['postgres.user_update'].assert_called_once_with(username='foo',
                                                                           superuser=None,
                                                                           encrypted=True,
                                                                           runas=None,
                                                                           inherit=None,
                                                                           rolepassword=None,
                                                                           port=None,
                                                                           replication=False,
                                                                           host=None,
                                                                           createroles=None,
                                                                           user=None,
                                                                           groups=None,
                                                                           maintenance_db=None,
                                                                           login=True,
                                                                           password=None,
                                                                           valid_until=None,
                                                                           createdb=None)

    def test_present__no_update(self):
        # test=True
        with patch.dict(postgres_user.__salt__, {'postgres.role_get': Mock(return_value={
                                                    'can create databases': False,
                                                    'can create roles': False,
                                                    'can login': False,
                                                    'can update system catalogs': False,
                                                    'connections': None,
                                                    'defaults variables': {},
                                                    'expiry time': None,
                                                    'inherits privileges': True,
                                                    'replication': False,
                                                    'superuser': False,
                                                 }),
                                                 'postgres.user_update': MagicMock()}):
            with patch.dict(postgres_user.__opts__, {'test': True}):
                ret = postgres_user.present('foo', login=False, replication=False)
                self.assertEqual(
                    ret,
                    {'comment': 'User foo is already present',
                     'changes': {}, 'name': 'foo', 'result': True}
                )
                self.assertEqual(self.salt_stub['postgres.user_update'].call_count, 0)

            # test=False
            ret = postgres_user.present('foo', login=False, replication=False)
            self.assertEqual(
                ret,
                {'comment': 'User foo is already present',
                 'changes': {}, 'name': 'foo', 'result': True}
            )
            self.assertEqual(self.salt_stub['postgres.user_update'].call_count, 0)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresGroupTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            'config.option': Mock(),
            'cmd.run_all': Mock(),
            'file.chown': Mock(),
            'file.remove': Mock(),
        }
        self.addCleanup(delattr, self, 'salt_stub')
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_extension: {},
            postgres_schema: {},
            postgres_group: {
                '__grains__': {'os_family': 'Linux'},
                '__salt__': self.salt_stub,
                '__opts__': {'test': False},
            }
        }

    def test_present__creation(self):
        # test=True
        with patch.dict(postgres_group.__salt__, {'postgres.role_get': Mock(return_value=None),
                                                  'postgres.group_create': MagicMock()}):
            with patch.dict(postgres_group.__opts__, {'test': True}):
                ret = postgres_group.present('foo')
                self.assertEqual(
                    ret,
                    {'comment': 'Group foo is set to be created',
                     'changes': {}, 'name': 'foo', 'result': None}
                )
                self.assertEqual(self.salt_stub['postgres.group_create'].call_count, 0)

            # test=False
            ret = postgres_group.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'The group foo has been created',
                 'changes': {}, 'name': 'foo', 'result': True}
            )
            self.salt_stub['postgres.group_create'].assert_called_once_with(superuser=None,
                                                                            replication=None,
                                                                            encrypted=True,
                                                                            runas=None,
                                                                            inherit=None,
                                                                            rolepassword=None,
                                                                            port=None,
                                                                            groupname='foo',
                                                                            host=None,
                                                                            createroles=None,
                                                                            user=None,
                                                                            groups=None,
                                                                            maintenance_db=None,
                                                                            login=None,
                                                                            password=None,
                                                                            createdb=None)

    def test_present__update(self):
        # test=True
        with patch.dict(postgres_group.__salt__, {'postgres.role_get': Mock(return_value={
                                                    'can create databases': False,
                                                    'can create roles': False,
                                                    'can login': False,
                                                    'can update system catalogs': False,
                                                    'connections': None,
                                                    'defaults variables': {},
                                                    'expiry time': None,
                                                    'inherits privileges': True,
                                                    'replication': False,
                                                    'superuser': False,
                                                 }),
                                                 'postgres.group_update': MagicMock()}):
            with patch.dict(postgres_group.__opts__, {'test': True}):
                ret = postgres_group.present('foo', login=True, replication=False)
                self.assertEqual(
                    ret,
                    {'comment': 'Group foo is set to be updated',
                     'changes': {'foo': {'login': True}}, 'name': 'foo', 'result': None}
                )
                self.assertEqual(self.salt_stub['postgres.group_update'].call_count, 0)

            # test=False
            ret = postgres_group.present('foo', login=True, replication=False)
            self.assertEqual(
                ret,
                {'comment': 'The group foo has been updated',
                 'changes': {'foo': {'login': True}}, 'name': 'foo', 'result': True}
            )
            self.salt_stub['postgres.group_update'].assert_called_once_with(superuser=None,
                                                                       replication=False,
                                                                       encrypted=True,
                                                                       runas=None,
                                                                       inherit=None,
                                                                       rolepassword=None,
                                                                       port=None,
                                                                       groupname='foo',
                                                                       host=None,
                                                                       createroles=None,
                                                                       user=None,
                                                                       groups=None,
                                                                       maintenance_db=None,
                                                                       login=True,
                                                                       password=None,
                                                                       createdb=None)

    def test_present__no_update(self):
        # test=True
        with patch.dict(postgres_group.__salt__, {'postgres.role_get': Mock(return_value={
                                                    'can create databases': False,
                                                    'can create roles': False,
                                                    'can login': False,
                                                    'can update system catalogs': False,
                                                    'connections': None,
                                                    'defaults variables': {},
                                                    'expiry time': None,
                                                    'inherits privileges': True,
                                                    'replication': False,
                                                    'superuser': False,
                                                 }),
                                                 'postgres.group_update': MagicMock()}):
            with patch.dict(postgres_group.__opts__, {'test': True}):
                ret = postgres_group.present('foo', login=False, replication=False)
                self.assertEqual(
                    ret,
                    {'comment': 'Group foo is already present',
                     'changes': {}, 'name': 'foo', 'result': True}
                )
                self.assertEqual(self.salt_stub['postgres.group_update'].call_count, 0)

            # test=False
            ret = postgres_group.present('foo', login=False, replication=False)
            self.assertEqual(
                ret,
                {'comment': 'Group foo is already present',
                 'changes': {}, 'name': 'foo', 'result': True}
            )
            self.assertEqual(self.salt_stub['postgres.group_update'].call_count, 0)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresExtensionTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            'config.option': Mock(),
            'cmd.run_all': Mock(),
            'file.chown': Mock(),
            'file.remove': Mock(),
        }
        self.addCleanup(delattr, self, 'salt_stub')
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_group: {},
            postgres_schema: {},
            postgres_extension: {
                '__grains__': {'os_family': 'Linux'},
                '__salt__': self.salt_stub,
                '__opts__': {'test': False},
            }
        }

    def test_present_failed(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch.dict(postgres_extension.__salt__, {
                    'postgres.create_metadata': Mock(side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [postgresmod._EXTENSION_TO_MOVE, postgresmod._EXTENSION_INSTALLED],

                    ]),
                    'postgres.create_extension': Mock(side_effect=[
                        False, False,
                    ])}):
            ret = postgres_extension.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'Failed to install extension foo',
                 'changes': {}, 'name': 'foo', 'result': False},
            )
            ret = postgres_extension.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'Failed to upgrade extension foo',
                 'changes': {}, 'name': 'foo', 'result': False}
            )

    def test_present(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch.dict(postgres_extension.__salt__, {
                    'postgres.create_metadata': Mock(side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [postgresmod._EXTENSION_INSTALLED],
                        [postgresmod._EXTENSION_TO_MOVE, postgresmod._EXTENSION_INSTALLED],

                    ]),
                    'postgres.create_extension': Mock(side_effect=[
                        True, True, True,
                    ])}):
            ret = postgres_extension.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'The extension foo has been installed',
                 'changes': {'foo': 'Installed'}, 'name': 'foo', 'result': True}
            )
            ret = postgres_extension.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'Extension foo is already present',
                 'changes': {}, 'name': 'foo', 'result': True}
            )
            ret = postgres_extension.present('foo')
            self.assertEqual(
                ret,
                {'comment': 'The extension foo has been upgraded',
                 'changes': {'foo': 'Upgraded'}, 'name': 'foo', 'result': True}
            )

    def test_presenttest(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch.dict(postgres_extension.__salt__, {
                    'postgres.create_metadata': Mock(side_effect=[
                        [postgresmod._EXTENSION_NOT_INSTALLED],
                        [postgresmod._EXTENSION_INSTALLED],
                        [postgresmod._EXTENSION_TO_MOVE, postgresmod._EXTENSION_INSTALLED],

                    ]),
                    'postgres.create_extension': Mock(side_effect=[
                        True, True, True,
                    ])}):
            with patch.dict(postgres_extension.__opts__, {'test': True}):
                ret = postgres_extension.present('foo')
                self.assertEqual(
                    ret,
                    {'comment': 'Extension foo is set to be installed',
                     'changes': {}, 'name': 'foo', 'result': None}

                )
                ret = postgres_extension.present('foo')
                self.assertEqual(
                    ret,
                    {'comment': "Extension foo is already present",
                     'changes': {}, 'name': 'foo', 'result': True}

                )
                ret = postgres_extension.present('foo')
                self.assertEqual(
                    ret,
                    {'comment': "Extension foo is set to be upgraded",
                     'changes': {}, 'name': 'foo', 'result': None}
                )

    def test_absent(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch.dict(postgres_extension.__salt__, {
                    'postgres.is_installed_extension': Mock(side_effect=[
                        True, False,
                    ]),
                    'postgres.drop_extension': Mock(side_effect=[
                        True, True,
                    ])}):
            ret = postgres_extension.absent('foo')
            self.assertEqual(
                ret,
                {'comment': 'Extension foo has been removed',
                 'changes': {'foo': 'Absent'}, 'name': 'foo', 'result': True}
            )
            ret = postgres_extension.absent('foo')
            self.assertEqual(
                ret,
                {'comment': (
                    'Extension foo is not present, '
                    'so it cannot be removed'),
                 'changes': {}, 'name': 'foo', 'result': True}

            )

    def test_absent_failed(self):
        '''
        scenario of creating upgrading extensions with possible schema and
        version specifications
        '''
        with patch.dict(postgres_extension.__opts__, {'test': False}):
            with patch.dict(postgres_extension.__salt__, {
                        'postgres.is_installed_extension': Mock(side_effect=[
                            True, True,
                        ]),
                        'postgres.drop_extension': Mock(side_effect=[
                            False, False,
                        ])}):
                ret = postgres_extension.absent('foo')
                self.assertEqual(
                    ret,
                    {'comment': 'Extension foo failed to be removed',
                     'changes': {}, 'name': 'foo', 'result': False}
                )

    def test_absent_failedtest(self):
        with patch.dict(postgres_extension.__salt__, {
                    'postgres.is_installed_extension': Mock(side_effect=[
                        True, True,
                    ]),
                    'postgres.drop_extension': Mock(side_effect=[
                        False, False,
                    ])}):
            with patch.dict(postgres_extension.__opts__, {'test': True}):
                ret = postgres_extension.absent('foo')
            self.assertEqual(
                ret,
                {'comment': 'Extension foo is set to be removed',
                 'changes': {}, 'name': 'foo', 'result': None}
            )


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PostgresSchemaTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        patcher = patch('salt.utils.path.which', Mock(return_value='/usr/bin/pgsql'))
        patcher.start()
        self.addCleanup(patcher.stop)
        self.salt_stub = {
            'config.option': Mock(),
            'cmd.run_all': Mock(),
            'file.chown': Mock(),
            'file.remove': Mock(),
        }
        self.addCleanup(delattr, self, 'salt_stub')
        return {
            postgres_database: {},
            postgres_user: {},
            postgres_extension: {},
            postgres_group: {},
            postgres_schema: {
                '__grains__': {'os_family': 'Linux'},
                '__salt__': self.salt_stub,
                '__opts__': {'test': False},
            }
        }

    def test_present_creation(self):
        with patch.dict(postgres_schema.__salt__, {'postgres.schema_get': Mock(return_value=None),
                                                   'postgres.schema_create': MagicMock()}):
            ret = postgres_schema.present('dbname', 'foo')
            self.assertEqual(
                ret,
                {'comment': 'Schema foo has been created in database dbname',
                 'changes': {'foo': 'Present'},
                 'dbname': 'dbname',
                 'name': 'foo',
                 'result': True}
                )
            self.assertEqual(self.salt_stub['postgres.schema_create'].call_count, 1)

    def test_present_nocreation(self):
        with patch.dict(postgres_schema.__salt__, {
                            'postgres.schema_get': Mock(return_value={'foo':
                                                                      {'acl': '',
                                                                       'owner': 'postgres'}
                                                                      }),
                            'postgres.schema_create': MagicMock()}):
            ret = postgres_schema.present('dbname', 'foo')
            self.assertEqual(
                ret,
                {'comment': 'Schema foo already exists in database dbname',
                 'changes': {},
                 'dbname': 'dbname',
                 'name': 'foo',
                 'result': True}
                )
            self.assertEqual(self.salt_stub['postgres.schema_create'].call_count, 0)

    def test_absent_remove(self):
        with patch.dict(postgres_schema.__salt__, {'postgres.schema_exists': Mock(return_value=True),
                                                   'postgres.schema_remove': MagicMock()}):
            ret = postgres_schema.absent('dbname', 'foo')
            self.assertEqual(
                ret,
                {'comment': 'Schema foo has been removed from database dbname',
                 'changes': {'foo': 'Absent'},
                 'dbname': 'dbname',
                 'name': 'foo',
                 'result': True}
                )
            self.assertEqual(self.salt_stub['postgres.schema_remove'].call_count, 1)

    def test_absent_noremove(self):
        with patch.dict(postgres_schema.__salt__, {'postgres.schema_exists': Mock(return_value=False),
                                                   'postgres.schema_remove': MagicMock()}):
            ret = postgres_schema.absent('dbname', 'foo')
            self.assertEqual(
                ret,
                {'comment': 'Schema foo is not present in database dbname,'
                            ' so it cannot be removed',
                 'changes': {},
                 'dbname': 'dbname',
                 'name': 'foo',
                 'result': True}
                )
            self.assertEqual(self.salt_stub['postgres.schema_remove'].call_count, 0)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)

# Import Salt Libs
import salt.modules.win_autoruns as win_autoruns

KEY = ['HKLM\\Software\\Microsoft\\Windows\\CurrentVersion\\Run',
       'HKLM\\Software\\Microsoft\\Windows\\CurrentVersion\\Run /reg:64',
       'HKCU\\Software\\Microsoft\\Windows\\CurrentVersion\\Run']


class WinAutorunsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_autoruns
    '''
    def setup_loader_modules(self):
        return {win_autoruns: {}}

    # 'list_' function tests: 1

    def test_list(self):
        '''
        Test if it enables win_autoruns the service on the server
        '''
        with patch('os.listdir', MagicMock(return_value=[])):
            ret = {KEY[0]: ['SALT'], KEY[1]: ['SALT'], KEY[2]: ['SALT']}
            mock = MagicMock(return_value='Windows 7')
            with patch.dict(win_autoruns.__grains__, {'osfullname': mock}):
                mock = MagicMock(return_value='SALT')
                with patch.dict(win_autoruns.__salt__, {'cmd.run': mock}):
                    self.assertDictEqual(win_autoruns.list_(), ret)
